(* Lese Versionsinfo aus einer Datei und ndere den Dateinamen einer
   anderen entsprechend:
   Aufruf:
     AddVersionToName <ExeFile> <InstallFile> [<Level>]
     ExeFile : Exe-Datei mit Versionsangabe
     InstallFile : zu ndernder Dateiname, z.B.
         pb-setup.exe ==> pb-setup-4.1.0201.exe
     Level (opt.): 1,2,3,4 (Standard: 4)
         Zu benutzende Versionslevel

    2007, J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   *)

program AddVersionToName;

{$APPTYPE CONSOLE}

uses
  System.SysUtils, Winapi.Windows;

const
  Space = ' ';
  Tab = #9;

  InfoNum = 12;
  InfoStr : array[1..InfoNum] of string = ('CompanyName','FileDescription','FileVersion',
          'InternalName','LegalCopyright','Comments','LegalTradeMarks','OriginalFileName',
          'ProductName','ProductVersion','PrivateBuild','SpecialBuild');

type
  TSystemID = (osWin32,osWin95,osWinNT);
  TSystemInfo = record
    SystemID          : TSystemID;
    MajorVersion,
    MinorVersion,
    Build             : integer;
    Info              : string[127];
    end;

{------------------------------------------------------------------}
(* fhrende Leerstellen und Tabs lschen *)
function RemSp (const S : string) : string;
var
  i : integer;
begin
  Result:=s; i:=1;
  while (i<=length(Result)) and ((Result[i]=Space) or (Result[i]=Tab)) do inc(i);
  delete(Result,1,pred(i));
  end;

{ --------------------------------------------------------------- }
(* fhrende Zeichen ergnzen *)
function AddChar (const S : string;
                  c       : char;
                  len     : integer) : string;
var
  i  : integer;
begin
  Result:=s;
  for i:=succ(length(Result)) to len do Result:=c+Result;
  end;

{ ------------------------------------------------------------------- }
(* Integer, Double oder String aus einem String s bis zum nchsten
   Komma lesen
   s wird um den verarbeiteten Teil gekrzt *)
function ReadNxtInt (var s   : String;
                     Del     : char;
                     Default : integer) : integer;
var
  n    : integer;
  i,ic : integer;
begin
  s:=RemSp(s); i:=pos (Del,s);
  if i=0 then i:=succ(length(s));
  val(copy(s,1,pred(i)),n,ic);
  if ic=0 then ReadNxtInt:=n
  else ReadNxtInt:=Default;
  delete(s,1,i);
  end;

{ ---------------------------------------------------------------- }
(* Integer-Zahl in String mit fhrenden Nullen umsetzen *)
function ZStrInt (x : int64;
                  n : integer) : string;
var
  s : string;
begin
  s:=IntToStr(abs(x));
  if x<0 then begin
    s:=AddChar(s,'0',n-1);
    s:='-'+s;
    end
  else s:=AddChar(s,'0',n);
  ZStrInt:=s;
  end;

{ --------------------------------------------------------------- }
(* Ausmaskieren der Erweiterung (ohne Punkt) *)
function GetExt (const name : string) : string;
var
  i,j : integer;
begin
  j:=length(Name);
  if j>0 then begin
    i:=j;
    while (i>0) and (Name[i]<>'.') do dec(i);
    if i=0 then Result:=''
    else Result:=copy(name,i+1,j-i);
    end
  else Result:='';
  end;

{ --------------------------------------------------------------- }
(* Dateinamenerweiterung entfernen (sucht letzten Punkt) *)
function DelExt (Name : string) : string;
var
  i,j : integer;
begin
  j:=length(Name);
  if j>0 then begin
    i:=j;
    while (i>0) and (Name[i]<>'.') do dec(i);
    if i>0 then delete (Name,i,j-i+1);
    end;
  Result:=Name;
  end;

{ ---------------------------------------------------------------- }
(* Suffix und Erweiterung zu Dateinamen hinzufgen *)
function AddNameSuffix (FName,Suffix,Ext : string) : string;
begin
  if (length(Ext)>0) and (Ext[1]<>'.') then Ext:='.'+Ext;
  Result:=DelExt(FName)+Suffix+Ext;
  end;

(* Suffix zu Dateinamen mit Extension hinzufgen *)
function InsertNameSuffix (FName,Suffix : string) : string;
begin
  Result:=AddNameSuffix(FName,Suffix,GetExt(FName));
  end;

{ ---------------------------------------------------------------- }
(*  Get Version Info from File *)
function GetVersion (Filename : string; var Version : string) : boolean;
var
  t       : string;
  n,Len,i : DWORD;
  Buf     : PByte;
  Value   : PChar;
  vvar    : PWordArray;
begin
  Result:=false;
  if (Win32Platform=VER_PLATFORM_WIN32_NT) and (Win32MajorVersion>=5) then begin   // use for Windows 2000 and newer
    n := GetFileVersionInfoSize(PChar(Filename),n);
    if n > 0 then begin
      Buf:=AllocMem(n);
      GetFileVersionInfo(PChar(Filename),0,n,Buf);     // get buffer
      // get translation code
      VerQueryValue(Buf,PChar('\VarFileInfo\Translation'),pointer(vvar),Len);
      t:=IntToHex(vvar^[0],4)+IntToHex(vvar^[1],4);
      for i := 1 to InfoNum do begin
        if VerQueryValue(Buf, PChar('\StringFileInfo\'+t+'\'+InfoStr[i]), Pointer(Value), Len) then begin
          if i=3 then Version:=value;
          end;
        end;
      FreeMem(Buf,n);
      Result:=true;
      end;
    end;
  end;

{ ---------------------------------------------------------------- }
var
  s,t  : string;
  n  : integer;
begin
  if ParamCount>1 then begin
    if ParamCount>2 then begin
      s:=ParamStr(3);
      n:=ReadNxtInt(s,'|',4);
      end
    else n:=4;  
    if FileExists(ParamStr(1)) and  GetVersion(ParamStr(1),t) then begin
      s:=IntToStr(ReadNxtInt(t,'.',0));
      if n>1 then s:=s+'.'+IntToStr(ReadNxtInt(t,'.',0));
      if n>2 then s:=s+'.'+ZStrInt(ReadNxtInt(t,'.',0),2);
      if n>3 then s:=s+ZStrInt(ReadNxtInt(t,'.',0),2);
      s:=InsertNameSuffix(ParamStr(2),'-'+s);
      if FileExists(ParamStr(2)) then begin
        DeleteFile(pchar(s));
        RenameFile(ParamStr(2),s);
        end;
      end;
    end;

end.
 